<?php

if (!defined('PLX_ROOT')) { exit; }

/**
 * Plugin Tinymce pour PluXml.
 *
 * Site de Tinymce: http://www.tinymce.com
 * La bibliothèque en ligne est disponible à : https://cdnjs.cloudflare.com/ajax/libs/tinymce/.
 * Elle est également disponible à : //cloud.tinymce.com/stable/tinymce.min.js?apikey='. Mais, il faut une "API KEY".
 * Pour intégrer la bibliothèque de Tinymce sur le serveur, voir https://www.tinymce.com/download/custom-builds/.
 *
 * Utilise le gestionnaire de médias de PluXml pour la sélection d'images dans Tinymce.
 * Dans le gestionnaire de médias, cliquer sur le lien de l'image ou son aperçu, pour insérer dans le document.
 * Les liens sont transformés en adressage absolu ( voir options : relative_urls, remove_script_host, document_base_url )
 *
 * Tinymce a un plugin codesample pour afficher le code source dans un article. Utilise la bibliothèque de http://prismjs.com
 *
 * Plugins externes pouvant être intégrés ultérieurement :
 *   - https://github.com/josh18/TinyMCE-FontAwesome-Plugin
 *   - ttps://github.com/RichGuk/syntaxhl
 *   - https://github.com/leevh/codemagic (plugin pour codemirror)
 * */
class tinyMCE extends plxPlugin {

	const TINYMCE_LIB = 'https://cdnjs.cloudflare.com/ajax/libs/tinymce/4.7.2/tinymce.min.js';

	const LOCAL_MINIFY_LIB = 'tinymce/js/tinymce/tinymce.min.js';
	// const LOCAL_MINIFY_LIB = 'tinymce/js/tinymce/tinymce.js';
	const BUNDLE = 'tinymce/js/tinymce/tinymce.full.min.js';

	// code et advcode sont incompatibles. Voir Tinymce. advcode nécessite une offre payante
	// manque : 'colorpicker, bbcode'
	const MCE_PLUGINS =
		'wordcount fullscreen save visualchars charmap '.
		'advlist autolink link image imagetools media lists hr anchor '.
		'emoticons searchreplace preview visualblocks nonbreaking '.
		'table contextmenu directionality template paste textcolor textpattern autoresize '.
		'help autosave code codesample';
	const BASIC_PLUGINS = 'wordcount fullscreen save';
	const MCE_PLUGINS_SITE = 'wordcount charmap autolink lists hr emoticons nonbreaking help';
	const CODESAMPLE_PATH = 'tinymce/js/tinymce/plugins/codesample/';

	public $weightProfils = array(PROFIL_ADMIN=>128, PROFIL_MANAGER=>64, PROFIL_MODERATOR=>32, PROFIL_EDITOR=>16, PROFIL_WRITER=>8);
	private $__scriptname = false;
	protected $__user = false;
	public $allow_comments = false;
	private $__primJS_plugin = false;
	public $template = false;

	public function __construct($default_lang) {

		# appel du constructeur de la classe plxPlugin (obligatoire)
		parent::__construct($default_lang);

		# droits pour accéder à la page config.php du plugin
		$this->setConfigProfil(PROFIL_ADMIN);

		$scriptname = basename(strtolower($_SERVER['SCRIPT_NAME']),'.php');

		if(
			isset($_SESSION['profil']) and
			!empty(intval($this->getParam($scriptname)) & $this->weightProfils[$_SESSION['profil']])
		) {
			$this->addHook('AdminFootEndBody', 'AdminFootEndBody');
			$this->__user = intval($_SESSION['profil']);
		}
		$this->__scriptname = $scriptname;

		$this->addHook('plxMotorConstruct', 'plxMotorConstruct');
		$this->addHook('AdminIndexFoot', 'AdminIndexFoot');
		$this->addHook('AdminMediasFoot', 'AdminMediasFoot');

		$this->addHook('plxMotorDemarrageEnd', 'plxMotorDemarrageEnd');
		$this->addHook('ThemeEndHead', 'ThemeEndHead');
		$this->addHook('ThemeEndBody', 'ThemeEndBody');

		// Récupère le template de l'article
		$this->addHook('plxMotorParseArticle', 'plxMotorParseArticle');
		// Récupère le template de la page statique
		$this->addHook('AdminStatic', 'AdminStatic');

		// A tester :
		$this->addHook('plxMotorAddCommentaire', 'plxMotorAddCommentaire');

	}

	public function onDelete() {
		# Pour anciennes version du plugin
		$filename = substr(__DIR__, 0, -strlen(__CLASS__)).'tinyMCE-pluxml.js';
		if (is_writable($filename)) {
			unlink($filename);
		}
	}

	public function get_context($plxMotor) {
		$this->plx_aConf = $plxMotor->aConf;
		$this->plx_racine = $plxMotor->racine;
		$this->plx_plugins = $this->plx_racine.$plxMotor->aConf['racine_plugins'];
		$this->plx_plugin_root = $this->plx_plugins.__CLASS__.'/';
		$this->plx_theme_folder = $plxMotor->aConf['racine_themes'].$plxMotor->style;
		$this->plx_theme = $this->plx_racine.$this->plx_theme_folder; /* don't end with '/' */
		// teste si le plugin codesample pour TinyMCE est sélectionné
		$this->__primJS_plugin = false;
		for($i=0; $i<5; $i++) {
			if(strpos($this->getParam('plugins'.$i), 'codesample') !== false) {
				if(array_key_exists('prismJS', $plxMotor->plxPlugins->aPlugins)) {
					$this->__primJS_plugin = $plxMotor->plxPlugins->aPlugins['prismJS'];
				} else {
					$this->__primJS_plugin_missing = true;
				}
				break;
			}
		}
	}

	public function initValues() {
		$formats = <<< EOT
{title: 'Image Left', selector: 'img', styles: {'float': 'left', 'margin': '0 10px 0 10px'}},
{title: 'Image Right', selector: 'img', styles: {'float': 'right', 'margin': '0 10px 0 10px'}},
{title: 'centrer', selector: 'p', classes: 'center'},
{title: 'Centrer blocs', selector: 'p, div', classes: 'center-blocks'},
{title: 'N° lignes', selector: 'pre', classes:'line-numbers'}
EOT;
		$initValues = array(
			'cdn'		=> '0', // plxPlugin->getParam('...') retourne une chaine de caractères
			'article'	=> 248, // 128+64+32+16+8
			'statique'	=> 0,
			'comment'	=> 248,
			'user'		=> 248,
			'codemirror'=> 128,
			// 'parametres_edittpl'=>128,
			'selector'	=> '#id_chapo, #id_content',
			'codemirror'=> 128,
			'css_theme'	=> 1,
			'styleformats'=> $formats,
			'plugins0'	=> $this::MCE_PLUGINS,
			'plugins1'	=> $this::BASIC_PLUGINS,
			'plugins2'	=> $this::BASIC_PLUGINS,
			'plugins3'	=> $this::BASIC_PLUGINS,
			'plugins4'	=> $this::BASIC_PLUGINS,
		);
		foreach($initValues as $field=>$value) {
			$this->setParam($field, $value, is_numeric($value) ? 'numeric' : 'string');
		}
	}

	public function get_skins() {
		$items = array(''=>'Par défaut');
		foreach(array('extras/skins/', 'tinymce/js/tinymce/skins/') as $folder) {
			if(is_dir(__DIR__.'/'.$folder)) {
				$files = glob(__DIR__.'/'.$folder.'*', GLOB_ONLYDIR);
				if (!empty($files)) {
					foreach ($files as $file1) {
						$value = substr(strrchr($file1, '/'), 1);
						$items[$folder.$value] = ucfirst($value);
					}
				}
				else { // Glob() ne marche pas chez certains hébergeurs comme Free.fr
					echo "<!--  Glob doesn't work! DirectoryIterator is using -->\n";
					$dir = new DirectoryIterator(__DIR__.$folder.'.');
					foreach ($dir as $fileinfo) {
						if ($fileinfo->isDir() && !$fileinfo->isDot()) {
							$filename = $fileinfo->getFilename();
							$items[$folder.$filename] = ucfirst($filename);
						}
					}
					asort($items);
				}
				break;
			}
		}
		return $items;
	}

	public function get_prism_css() {
		$arr = array('' => $this->getLang('NOTHING'));
		$mask = __DIR__.'/'.$this::CODESAMPLE_PATH.'css/';
		$offset = strlen($mask);
		$files = glob($mask.'*.css');
		foreach($files as $f) {
			$value = substr($f, $offset);
			$caption = preg_replace('@^prism-?(\w*)\.css$@', '$1', $value);
			if($caption == '') {
				$caption = $this->getLang('DEFAULT');
			}
			$arr[$value] = ucfirst($caption);
		}
		return $arr;
	}

	private function __print_mce_link_list_option() {
		//link_list				: <?php $this->__build_link_list_option();
		// https://www.tinymce.com/docs/plugins/link/#link_list

		global $plxAdmin;

		$sub_links = array();
		$top_lines1 = array();

		if(!empty($plxAdmin->aCats)) {
			$lines = array();
			foreach($plxAdmin->aCats as $k=>$v) {
				$title = addslashes($v['name']);
				// $value = $plxAdmin->urlRewrite('?categorie'.intval($k).'/'.$v['url']);
				$value = 'index.php?categorie'.intval($k).'/'.$v['url'];
				$lines[] = <<< CATS
					{ title: '$title', value: '$value' }
CATS;
			}
			if(!empty($lines)) {
				$menu = implode(",\n", $lines);
				$title = addslashes(L_MENU_CATEGORIES);
				$top_lines1[] = <<< CATEGORIES
			{
				title	: '$title',
				menu	: [
$menu
				]
			}
CATEGORIES;
			}
		}

		if(! empty($plxAdmin->aTags)) {
			// voir plxShow::tagList()
			$today = date('YmdHi');
			$tagsList = array();
			foreach($plxAdmin->aTags as $idArt => $infosArt) {
				if(! empty($infosArt['active'] and ! empty(trim($infosArt['tags']))) and ($infosArt['date'] <= $today)) {
					$tagsArt = array_map('trim', explode(',', $infosArt['tags']));
					foreach($tagsArt as $tag) {
						if(array_key_exists($tag, $tagsList)) {
							$tagsList[$tag]++;
						} else {
							$tagsList[$tag] = 1;
						}
					}
				}
			}

			if(! empty($tagsList)) {
				uksort($tagsList, function($item1, $item2) {
					return strcasecmp($item1, $item2);
				});
				$lines = array();
				foreach($tagsList as $k=>$v) {
					$title = addslashes($k).' ('.$v.')';
					// $value = $plxAdmin->urlRewrite('?tag/'.plxUtils::title2url($k));
					$value = 'index.php?tag/'.plxUtils::title2url($k);
					$lines[] = <<< TAGS
					{ title: '$title', value: '$value' }
TAGS;
				}
				if(!empty($lines)) {
					$title = addslashes(L_ARTICLE_TAGS_FIELD);
					$menu = implode(",\n", $lines);
					$top_lines1[] = <<< TAGS
			{
				title	: '$title',
				menu	: [
$menu
				]
			}
TAGS;
				}
			}
		}

		if(!empty($plxAdmin->aStats)) {
			$statsGroups = array();
			foreach($plxAdmin->aStats as $idStat=>$infosStat) {
				if(! empty($infosStat['active']) and ($infosStat['date_update'] <= $today)) {
					$group = strtolower(trim($infosStat['group']));
					if(empty($group)) {
						$group = 'zzzzzzzzzzz';
					}
					if(!array_key_exists($group, $statsGroups)) {
						$statsGroups[$group] = array();
					}
					// $link = $plxAdmin->urlRewrite("?static$idStat/${infosStat['url']}");
					$link = "index.php?static$idStat/${infosStat['url']}";
					$statsGroups[$group][ucfirst($infosStat['name'])] = $link;
				}
			}
			ksort($statsGroups);

			$lines = array();
			foreach($statsGroups as $group=>$options) {
				ksort($options);
				$sub_lines = array();
				foreach($options as $title=>$value) {
					$title1 = addslashes($title);
					$sub_lines[] = <<< SUB_LINE
					{ title: '$title1', value: '$value' }
SUB_LINE;
				}
				if(!empty($sub_lines)) {
					$menu = implode(",\n", $sub_lines);
					if(count($statsGroups) == 1) {
						$lines[] = $menu;
					} else {
						$title = addslashes(($group != 'zzzzzzzzzzz') ? $group : ucfirst($this->getLang('OTHERS_PAGES')));
						$lines[] = <<< STATS_GROUP
				{
					title	: '$title',
					menu	: [
$menu
					]
				}
STATS_GROUP;
					}
				}
			}
			if(!empty($lines)) {
				$menu = implode(",\n", $lines);
				$title = addslashes(L_MENU_STATICS);
				$top_lines1[] = <<< STATICS
			{
				title	: '$title',
				menu	: [
$menu
				]
			}

STATICS;
			}

			if(!empty($top_lines1)) {
				$list_link = implode(",\n", $top_lines1);
				echo <<< LINK_LIST
		link_list				: [
$list_link
		],

LINK_LIST;
			}
		}
	}

	private function get_prism_css_url($default=false) {
		$result = false;
		$css = $this->getParam('prism_css');
		if(!empty($css)) {
			$filename = $this::CODESAMPLE_PATH.'css/'.$css;
			if(file_exists(__DIR__.'/'.$filename)) {
				$result = $this->plx_plugin_root.$filename;
			}
		}
		if(empty($result) and $default) {
			//$result = $this::CODESAMPLE_CSS;
			$result = $this->plx_plugin_root.$this::CODESAMPLE_PATH.'css/prism.css';
		}
		return $result;
	}

	private function __print_codesample_params() {

		if(!empty($this->__primJS_plugin)) {
			// Le plugin primsJS est actif.
			$filename = substr(__DIR__, 0, -strlen(__CLASS__)).'prismJS/prism/prism.js';
			if(file_exists($filename)) {
				$lines = file($filename);
				if(preg_match('@languages=([^&]*)@', $lines[0], $matches)) {
					$languages = explode('+', $matches[1]);
					$uppercases = explode(' ', 'php css json yaml sql http');
					$captions = array(
						'markup'		=> 'HTML/XML',
						'csharp'		=> 'C#',
						'cpp'			=> 'C++',
						'apacheconf'	=> 'Apache',
					);
					$lines = array();
					foreach($languages as $lang) {
						if(in_array($lang, $uppercases)) {
							$caption = strtoupper($lang);
						} elseif(array_key_exists($lang, $captions)) {
							$caption = $captions[$lang];
						} else {
							$caption = ucfirst($lang);
						}
						$lines[] = "text: '$caption', value: '$lang'";
					}
					$sep = <<< SEP
},
			{
SEP;
					$content = '{'.implode($sep, $lines).'}';
					echo <<< EOT
		codesample_languages	: [
			$content
		],
		codesample_autoloader_path: {$this->plx_plugin_root}prism/components/',

EOT;
				}
			}
		} else {
			$css_url = $this->get_prism_css_url();
			if(!empty($css_url)) {
				echo <<< CSS
		codesample_content_css	: '$css_url',

CSS;
			}
		}
	}

	private function __print_mce_external_plugins_options() {
		global $plxAdmin;

		$mask_user = $this->weightProfils[$this->__user];
		$lines = array();
		foreach(explode(' ', 'codemirror codemagic') as $plugin) {
			$filename = $plugin.'/tinymce/plugin.min.js';
			if(
				array_key_exists($plugin, $plxAdmin->plxPlugins->aPlugins) and
				!empty(intval($this->getParam($plugin)) & $mask_user) and
				file_exists(PLX_PLUGINS.$filename)
			) {
				// adresse absolue requise !
				$url = $plxAdmin->racine.$plxAdmin->aConf['racine_plugins'].$filename;
				$lines[] = <<< PLUGIN
				$plugin	: '$url'
PLUGIN;
			}
		}
		if(!empty($lines)) {
			$plugins = implode(",\n", $lines);
			echo <<< START_EXTERNAL_PLUGINS
		external_plugins	: {
$plugins
		},

START_EXTERNAL_PLUGINS;
		}
	}

	public function extract_css($item) {
		return preg_replace($this->searches, $this->replaces, trim($item));
	}

	private function __print_content_css() {
		// https://www.tinymce.com/docs/configure/content-appearance/#content_css

		if(
			!empty($this->getParam('css_theme')) and
			in_array($this->__scriptname, explode(' ', 'article statique'))
		) {
			$themeRoot = preg_replace('@[^/]+/[^/]+$@', '', __DIR__).$this->plx_theme_folder;
			$header_filename = $themeRoot.'/header.php';
			$this->searches = array(
				'@^.*href="<\?php\s+\$plxShow->template\(\);?\s+\?>([^"]*)".*$@i',
				'@^.*\$plxShow->pluginsCss\(\).*$@i'
			);
			$this->replaces = array(
				$this->plx_theme."$1",
				$this->plx_plugins.'site.css'
			);
			$patternCss = 'plugins';
			if(!empty($this->template)) {
				$templates = glob($themeRoot.'/'.$this->template.'{.min.css,.css}', GLOB_BRACE);
				if(!empty($templates)) {
					sort($templates);
					$this->searches[] = '@^.*\$plxShow->templateCss\(\).*$@i';
					$this->replaces[] = $this->plx_theme.substr($templates[count($templates) - 1], strlen($themeRoot));
					$patternCss = '(?:plugins|template)';
				}
			}
			$pattern = '@(?:<link\s[^>]*rel="stylesheet"|\$plxShow->'.$patternCss.'css\(\))@i';
			$buf = array_values(preg_grep(
				$pattern,
				file($header_filename)
			));
			$links = array_map(
				array($this, 'extract_css'),
				$buf
			);
			$sep = <<< SEP
',
			'
SEP;
			$values = implode($sep, $links);
			echo <<< CONTENT_CSS
		content_css				: [
			'$values'
		],

CONTENT_CSS;
		}
	}

	private function __print_mce_launch($site=false) {
		global $plxAdmin;
?>
	<script type="text/javascript">
	tinymce.init({
<?php	/* ---------- site only ------------- */
		if(!empty($site)) {
?>
		selector			: '#id_content',
		plugins				: '<?php echo $this::MCE_PLUGINS_SITE; ?>',
		toolbar				:
			'undo redo | bold italic styleselect alignleft aligncenter alignright alignjustify | ' +
			'bullist numlist outdent indent | emoticons charmap visualchars',
		menubar				: false,
<?php	/* ---------- admin only ------------ */
		} else {
		// selectors CSS for blocks to apply TinyMCE
		$selector = trim($this->getParam('selector'));
		if(empty($selector)) $selector = 'textarea';

		// https://www.tinymce.com/docs/configure/content-formatting/#style_formats_merge
?>
		selector			: '<?php echo $selector; ?>',
		plugins				: '<?php echo $this->getParam('plugins'.$this->__user); ?>',
		toolbar				:
			'save undo redo | bold italic styleselect ' +
			'alignleft aligncenter alignright alignjustify | ' +
			'bullist numlist outdent indent | ' +
			'link image media anchor emoticons | ' +
			'forecolor backcolor | ' +
			' charmap visualchars fullscreen code codesample',
		// menubar			: 'insert tools restoredraft', // 2017-08-01
		images_reuse_filename	: true,
		file_browser_callback_types	: 'file image codesample',
		imagetools_toolbar	: 'rotateleft rotateright | flipv fliph | editimage imageoptions',
		image_advtab		: true,
		image_description	: true,
		image_title			: true,
		image_caption		: true,
		image_class_list	: [
			{ title	: 'Aucune',			value	: ''		   },
			{ title	: 'Galerie n°1',	value	: 'lightbox-1' },
			{ title	: 'Galerie n°2',	value	: 'lightbox-2' },
			{ title	: 'Galerie n°3',	value	: 'lightbox-3' }
		],
		media_alt_source		: false,
		paste_data_images		: true,
		save_onsavecallback		: saveDocument,
		file_picker_callback	: myFile_picker_callback,
<?php
			echo <<< DOCUMENT_BASE_URL
		document_base_url		: '{$this->plx_racine}',

DOCUMENT_BASE_URL;
			// Pictures are not managing in the front end
			// used by imagetools plugin from Tinymce
			// https://www.tinymce.com/docs/advanced/php-upload-handler
			$images_upload_url = $this->plx_plugin_root.'postAcceptor.php';
			// used by postAcceptor.php
			$path1 = ($plxAdmin->aConf['userfolders'] and ($this->__user == PROFIL_WRITER)) ? $this->__user.'/' : '';
			$images_upload_base_path = $plxAdmin->aConf['medias'].$path1;
			$files_root = substr(dirname(__FILE__), 0, -strlen($plxAdmin->aConf['racine_plugins'].__CLASS__));
			$_SESSION['tinyMCE_medias'] = $files_root.$plxAdmin->aConf['medias'].$path1;
			echo <<< IMAGES_UPLOAD
		images_upload_url		: '$images_upload_url',
		images_upload_base_path	: '$images_upload_base_path',

IMAGES_UPLOAD;
			$this->__print_codesample_params();
			$this->__print_mce_link_list_option();
			$this->__print_content_css();
			$this->__print_mce_external_plugins_options();

			// https://www.tinymce.com/docs/configure/content-appearance/#body_class
			// https://www.tinymce.com/docs/configure/content-appearance/#body_id: Pas utilisé par le thème par défaut
			$body_classes = array(
				'article'		=> 'container',
				'statique'		=> 'static-page',
				'comment'		=> 'comment',
				'new_comment'	=> 'body-article', // A vérifier
				'user'			=> 'admin-page' // A vérifier
			);
			if(array_key_exists($this->__scriptname, $body_classes)) {
				$body_class = $body_classes[$this->__scriptname];
				echo <<< BODY_CLASS
		body_class				: '$body_class',

BODY_CLASS;
			}
			if(!empty($this->getParam('style_formats_merge'))) {
				// https://www.tinymce.com/docs/configure/content-formatting/#style_formats_merge
				echo <<< STYLE_FORMATS_MERGE
		style_formats_merge		: true,

STYLE_FORMATS_MERGE;
			}
		} /* -------- end of admin only -------- */
/* ----------- common parts ---------- */
		$skin = $this->getParam('skin');
		if (! empty($skin) and is_readable(__DIR__.'/'.$skin)) {
			echo <<< SKIN_URL
		skin_url				: '{$this->plx_plugin_root}$skin',

SKIN_URL;
		}

		// checkout for language
		$language = $this->default_lang;
		if($language != 'en') {
			$i18N = array('fr'=>'fr_FR', 'pt'=>'pt_PT');
			if (array_key_exists($language, $i18N)) {
				$language = $i18N[$language];
			}
			echo <<< LANGUAGE
		language				: '$language',

LANGUAGE;
			if(true or !empty($this->getParam('cdn'))) {
				$language_url =
					$this->plx_plugin_root.
					pathinfo($this::LOCAL_MINIFY_LIB, PATHINFO_DIRNAME).
					"/langs/$language.js";
				echo <<< LANGUAGE_URL
		language_url			: '$language_url',

LANGUAGE_URL;
			}
		}
		# styleformats param is stored in javascript context. Don't ##json_encode## again !
		$style_formats = trim($this->getParam('styleformats'));
		if(!empty($style_formats)) {
			echo <<< STYLE_FORMATS
		style_formats			: [
$style_formats
		],

STYLE_FORMATS;
		}
		/* http://www.tinymce.com/wiki.php/Configuration:entity_encoding */
?>
		entities				:
			'160,nbsp,38,amp,34,quot,162,cent,169,copy,174,reg,' +
			'8482,trade,8240,permil,60,lt,62,gt,8804,le,8805,ge,' +
			'176,deg,8722,minus',
		paste_as_text			: true,
		nonbreaking_force_tab	: true
	});
	</script>
<?php
	}

// ------------- Hooks start here ----------------------------------
	public function plxMotorConstruct() {
		echo '<?php $this->plxPlugins->aPlugins["'.__CLASS__.'"]->get_context($this); ?>';
	}

	public function plxMotorParseArticle() {
		echo '<?php $this->plxPlugins->aPlugins["'.__CLASS__.'"]->template = basename($art["template"], ".php"); ?>';
	}

	public function AdminStatic() {
		echo '<?php $plxAdmin->plxPlugins->aPlugins["'.__CLASS__.'"]->template = basename($template, ".php"); ?>';
	}
	public function AdminFootEndBody($site=false) {
		if(empty($site)) {
?>
		<!-- ========= plugin: <?php echo __CLASS__; ?> $this->__scriptname : <?php echo $this->__scriptname; ?> ========== -->
		<div id="tinymce-spinner">
			<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" version="1.1" width="160" height="160">
			  <g>
			    <line id="line" x1="10" y1="80" x2="32" y2="80" stroke="red" stroke-width="8" style="stroke-linecap:round"/>
			    <use xlink:href="#line" transform="rotate( 30,80,80)" style="opacity:.0833"/>
			    <use xlink:href="#line" transform="rotate( 60,80,80)" style="opacity:.1660"/>
			    <use xlink:href="#line" transform="rotate( 90,80,80)" style="opacity:.2500"/>
			    <use xlink:href="#line" transform="rotate(120,80,80)" style="opacity:.3333"/>
			    <use xlink:href="#line" transform="rotate(150,80,80)" style="opacity:.4166"/>
			    <use xlink:href="#line" transform="rotate(180,80,80)" style="opacity:.5000"/>
			    <use xlink:href="#line" transform="rotate(210,80,80)" style="opacity:.5833"/>
			    <use xlink:href="#line" transform="rotate(240,80,80)" style="opacity:.6666"/>
			    <use xlink:href="#line" transform="rotate(270,80,80)" style="opacity:.7500"/>
			    <use xlink:href="#line" transform="rotate(300,80,80)" style="opacity:.8333"/>
			    <use xlink:href="#line" transform="rotate(330,80,80)" style="opacity:.9166"/>

			    <animateTransform attributeName="transform" attributeType="XML"
					type="rotate" begin="0s" dur="2s" repeatCount="indefinite" calcMode="discrete"
					keyTimes="0;.0833;.166;.25;.3333;.4166;.5;.5833;.6666;.75;.8333;.9166;1"
					values="  0,80,80;
							 30,80,80;
							 60,80,80;
							 90,80,80;
							120,80,80;
							150,80,80;
							180,80,80;
							210,80,80;
							240,80,80;
							270,80,80;
							300,80,80;
							330,80,80;
							360,80,80"/>
			  </g>
			</svg>
		</div>
<?php
		}

		if(
			!empty($this->getParam('cdn')) or
			!is_readable(__DIR__.'/'.$this::LOCAL_MINIFY_LIB)
		) {
			// loading library for TinyMCE via CDN
			$src = $this::TINYMCE_LIB;
		} else {
			// loading library for TinyMCE locally
			$src = $this->plx_plugin_root;
			// Ne pas utiliser le bundle côté site: conflit avec prismJS
			$src .= (!$site && file_exists(__DIR__.'/'.$this::BUNDLE)) ? $this::BUNDLE : $this::LOCAL_MINIFY_LIB;
		}
?>
	<script type="text/javascript" src="<?php echo $this->plx_plugin_root; ?>tinyMCE.js"></script>
	<script type="text/javascript" src="<?php echo $src; ?>"></script>
<?php
		$this->__print_mce_launch($site); // Lance Tinymce avec ses options
	}

/* ---------------------- for using core/admin/medias.php as medias manager ----------------- */
	public function AdminMediasFoot($params=false) {
		global $plxAdmin;

		if (defined('PLX_VERSION')) {
			$versionPluxml = PLX_VERSION;
		} else {
			$versionPluxml = strval($plxAdmin->version);
		}
		$idTable = (version_compare($versionPluxml, '5.4', '<')) ? 'list-files' : 'medias-table'; ?>
		<script type="text/javascript"
			src="<?php echo $this->plx_plugin_root.__CLASS__; ?>.js"
			data-medias-id-table="<?php echo $idTable; ?>"
			data-pluxml-version="<?php echo $versionPluxml; ?>"
			data-medias-type="media"
		></script>
<?php
	}

/* --------------- for using core/admin/index.php as articles manager like medias manager ------- */
	public function AdminIndexFoot() {

		if (defined('PLX_VERSION')) {
			$versionPluxml = PLX_VERSION;
		} else {
			global $plxAdmin;
			$versionPluxml = strval($plxAdmin->version);
		}
		$idTable = (version_compare($versionPluxml, '5.4', '<')) ? 'table' : 'articles-table'; ?>
		<script type="text/javascript"
			src="<?php echo $this->plx_plugin_root.__CLASS__; ?>.js"
			data-medias-id-table="<?php echo $idTable; ?>"
			data-pluxml-version="<?php echo $versionPluxml; ?>"
			data-medias-type="file"
		></script>
<?php
	}

	/* -------------- Edition des commentaires côté site ----------- */
	public function plxMotorDemarrageEnd() {
		$this->allow_comments = false;
		if(!empty($this->getParam('comment_site'))) {
			$code = <<< 'CODE'
<?php
	$this->plxPlugins->aPlugins['##CLASS##']->allow_comments = (
		$this->mode === 'article' and
		!empty($this->plxRecord_arts) and
		$this->aConf['allow_com'] and
		$this->plxRecord_arts->f('allow_com')
	);
?>
CODE;
			echo str_replace('##CLASS##', __CLASS__, $code);
		}
	}

	public function ThemeEndHead() {
		if(!empty($this->__primJS_plugin_missing)) {
			$href = $this->get_prism_css_url(true);
			echo <<< LINK
	<link rel="stylesheet" type="text/css" href="$href" />

LINK;
		}

	}

	public function ThemeEndBody() {
		if(!empty($this->__primJS_plugin_missing)) {
			$src = $this->plx_plugin_root.$this::CODESAMPLE_PATH;
			$path = $this::CODESAMPLE_PATH;
			echo <<< SCRIPT
	<script type="text/javascript" src="{$src}clipboard.min.js"></script>
	<script type="text/javascript" src="{$src}prism.js"></script>

SCRIPT;
		}
		if($this->allow_comments) {
			$this->AdminFootEndBody(true);
		}
	}

	public function plxMotorAddCommentaire() {
		if($this->allow_comments) {
			// Adaptation de PlxMotor::addCommentaire($content) pour accepter les balises HTML dans les commentaires
			$code = <<< 'CODE'
<?php
	$content['content'] = preg_replace('@<(\s*(?:script|\?php))@i', '&lt;$1', html_entity_decode($content['content']));
?>
CODE;
			echo $code;
		}
	}

}

?>